; milliGun/theme.jl

;; Copyright (C) 1999 Ryan Lovett <ryan@ocf.berkeley.edu>
;; Copyright (C) 2000 Pierre Baillargeon <pierreba@globetrotter.net>

;; This theme is derived from microGUI, with customizable amount of blue,
;; centered title, close button alone on the left, nicer transient windows,
;; and bigger image sizes (x1.5) for a nice 1600x1200 style.

(defgroup milliGun "milliGun Theme"
  :group appearance)

(defcustom milliGun:percent-blue 90
  "Percent of unused title in blue."
  :group (appearance milliGun)
  :type number
  :range (0 . 100))

(defcustom milliGun:percent-green 50
  "Percent of unused transient title in green."
  :group (appearance milliGun)
  :type number
  :range (0 . 100))

(defcustom milliGun:center-title t
  "Center text in title bar."
  :group (appearance milliGun)
  :type boolean)

(let*


    ;; Update window title pixel length
    ((title-width
      (lambda (w)
	(let
	    ((w-width (car (window-dimensions w))))
	  (max 0 (min (- w-width 100) (text-width (window-name w)))))))

     (w-width (lambda (w) (car (window-dimensions w))))

     (w-height (lambda (w) (cdr (window-dimensions w))))

     (green-height (lambda (w) (max 0 (/ (*    (- (w-height w)                     56  )    milliGun:percent-green) 100))))

     (blue-width   (lambda (w) (max 0 (/ (* (/ (- (w-width w)  (+ (title-width w) 168) ) 2) milliGun:percent-blue)  100))))

     (left-blue-width   (lambda (w) (+ (title-width w) (blue-width w))))

     (select-blue-width   (lambda (w) (if milliGun:center-title (blue-width w) (left-blue-width w))))


     ;; called after the WM_NAME property changes
     (update-text-width
      (lambda (w)
	(if (eq (window-get w 'current-frame-style) 'milliGun)
	    (rebuild-frame w))))
     
     (bottom-images (make-image "bottom.png"))

     (top-left-images (list (make-image "top_left_inactive.png")
			    (make-image "top_left.png")))

     (top-left-shaded-images (list (make-image "top_left_inactive_s.png")
				   (make-image "top_left_s.png")))

     (top-blue-images (list (make-image "top_blue_inactive.png")
			    (make-image "top_blue.png")))

     (top-curves-images (list (make-image "top_curves_inactive.png")
			      (make-image "top_curves.png")))

     (top-right-curves-images (list (make-image "top_right_curves_inactive.png")
			      (make-image "top_right_curves.png")))

     (top-grey-images (make-image "top_grey.png"))

     (top-right-images (list (make-image "top_right_inactive.png")
                             (make-image "top_right.png")))

     (top-right-shaded-images (list (make-image "top_right_inactive_s.png")
				    (make-image "top_right_s.png")))

     (bottom-left-images (make-image "bl.png"))

     (bottom-right-images (make-image "br.png"))

     (close-images (list (make-image "close_normal.png")
			 (make-image "close_active.png") nil
			 (make-image "close_clicked.png")))

     (maximize-images (list (make-image "maximize_normal.png")
			    (make-image "maximize_active.png") nil
			    (make-image "maximize_clicked.png")))

     (iconify-images (list (make-image "minimize_normal.png")
			   (make-image "minimize_active.png") nil
			   (make-image "minimize_clicked.png")))

     (menu-images (list (make-image "menu_normal.png")
			(make-image "menu_active.png") nil
			(make-image "menu_clicked.png")))

     (left-images (make-image "left.png"))

     (right-images (make-image "right.png"))

     (top-images (flip-image-diagonally (copy-image left-images)))

     (t-close-images (list (make-image "t_close_normal.png")
			   (make-image "t_close_active.png") nil
			   (make-image "t_close_clicked.png")))

     (t-right-curves-images (list (make-image "t_curves_inactive.png")
			      (make-image "t_curves.png")))

     (t-right-green-images (list (make-image "t_right_green_inactive.png")
			      (make-image "t_right_green.png")))
     (t-left-images (make-image "t_left.png"))

     (t-right-images (make-image "t_right.png"))

     (t-top-images (make-image "t_top.png"))

     (t-bottom-images (make-image "t_bottom.png"))

     (t-top-left-images (make-image "t_top_left.png"))

     (t-bottom-left-images (make-image "t_bottom_left.png"))

     (t-top-right-images (list (make-image "t_top_right_inactive.png")
                               (make-image "t_top_right.png")))

     (t-top-right-shaded-images (list (make-image "t_top_right_inactive_s.png")
				       (make-image "t_top_right_s.png")))

     (t-bottom-right-images (make-image "t_bottom_right.png"))

     (t-bottom-right-shaded-images (make-image "t_bottom_right_s.png"))

     ;; frame layout

     (frame `(
	      ((background . ,top-left-images)
	       (left-edge . -6)
	       (top-edge . -28)
	       (class . top-left-corner))

	      ;; delete button
	      ((background . ,close-images )
	       (top-edge . -28)
	       (left-edge . 0)
	       (class . close-button))

	      ;; menu button
	      ((background . ,menu-images)
	       (top-edge . -28)
	       (left-edge . 28)
	       (class . menu-button))

	      ;; iconify button
	      ((background . ,iconify-images)
	       (right-edge . 28)
	       (top-edge . -28)
	       (class . iconify-button))

	      ;; maximize button
	      ((background . ,maximize-images)
	       (right-edge . 0)
	       (top-edge . -28)
	       (class . maximize-button))

	      ;; top left blue
	      ((background . ,top-blue-images)
	       (foreground . "black")
	       (text . ,(if milliGun:center-title nil window-name))
	       (x-justify . left)
	       (y-justify . top)
	       (top-edge . -28)
	       (left-edge . 56)
	       (width . ,(lambda (w) (select-blue-width w)))
	       (class . title))

	      ;; top left curves
	      ((background . ,top-curves-images)
	       (left-edge . ,(lambda (w) (+ (select-blue-width w) 56)))
	       (top-edge . -28)
	       (class . title))

	      ;; top right blue
	      ((background . ,top-blue-images)
	       (top-edge . -28)
	       (right-edge . 56)
	       (width . ,(lambda (w) (blue-width w)))
	       (class . title))

	      ;; top right curves
	      ((background . ,top-right-curves-images)
	       (right-edge . ,(lambda (w) (+ (blue-width w) 56)))
	       (top-edge . -28)
	       (class . title))

	      ;; top grey
	      ((background . ,top-grey-images)
	       (foreground . "black")
	       (text . ,(if milliGun:center-title window-name nil))
	       (x-justify . center)
	       (y-justify . top)
	       (top-edge . -28)
	       (left-edge . ,(lambda (w) (+ (select-blue-width w) 74)))
	       (right-edge . ,(lambda (w) (+ (blue-width w) 74)))
	       (class . title))

	      ;; left border
	      ((background . ,left-images)
	       (left-edge . -6)
	       (top-edge . 8)
	       (bottom-edge . 0)
	       (class . left-border))

	      ;; top-right corner
	      ((background . ,top-right-images)
	       (right-edge . -6)
	       (top-edge . -28)
	       (class . top-right-corner))

	      ;; right border
	      ((background . ,right-images)
	       (right-edge . -6)
	       (top-edge . 8)
	       (bottom-edge . 0)
	       (class . right-border))

	      ;; bottom border
	      ((background . ,bottom-images)
	       (left-edge . 0)
	       (right-edge . 0)
	       (bottom-edge . -6)
	       (class . bottom-border))

	      ;; bottom-left corner
	      ((background . ,bottom-left-images)
	       (left-edge . -6)
	       (bottom-edge . -6)
	       (class . bottom-left-corner))

	      ;; bottom-right corner
	      ((background . ,bottom-right-images)
	       (right-edge . -6)
	       (bottom-edge . -6)
	       (class . bottom-right-corner))
					 ))

       (shaded-frame `(
	      ((background . ,top-left-shaded-images)
	       (left-edge . -6)
	       (top-edge . -28)
	       (height . 28)
	       (class . top-left-corner))

	      ;; delete button
	      ((background . ,close-images )
	       (top-edge . -28)
	       (left-edge . 0)
	       (class . close-button))

	      ;; menu button
	      ((background . ,menu-images)
	       (top-edge . -28)
	       (left-edge . 28)
	       (class . menu-button))

	      ;; iconify button
	      ((background . ,iconify-images)
	       (right-edge . 28)
	       (top-edge . -28)
	       (class . iconify-button))

	      ;; maximize button
	      ((background . ,maximize-images)
	       (right-edge . 0)
	       (top-edge . -28)
	       (class . maximize-button))

	      ;; top left blue
	      ((background . ,top-blue-images)
	       (foreground . "black")
	       (text . ,(if milliGun:center-title nil window-name))
	       (x-justify . left)
	       (y-justify . top)
	       (top-edge . -28)
	       (left-edge . 56)
	       (width . ,(lambda (w) (select-blue-width w)))
	       (class . title))

	      ;; top left curves
	      ((background . ,top-curves-images)
	       (left-edge . ,(lambda (w) (+ (select-blue-width w) 56)))
	       (top-edge . -28)
	       (class . title))

	      ;; top blue
	      ((background . ,top-blue-images)
	       (top-edge . -28)
	       (right-edge . 56)
	       (width . ,(lambda (w) (blue-width w)))
	       (class . title))

	      ;; top right curves
	      ((background . ,top-right-curves-images)
	       (right-edge . ,(lambda (w) (+ (blue-width w) 56)))
	       (top-edge . -28)
	       (class . title))

	      ;; top grey
	      ((background . ,top-grey-images)
	       (foreground . "black")
	       (text . ,(if milliGun:center-title window-name nil))
	       (x-justify . center)
	       (y-justify . top)
	       (top-edge . -28)
	       (left-edge . ,(lambda (w) (+ (select-blue-width w) 74)))
	       (right-edge . ,(lambda (w) (+ (blue-width w) 74)))
	       (class . title))

	      ;; top-right corner
	      ((background . ,top-right-shaded-images)
	       (right-edge . -6)
	       (top-edge . -28)
	       (height . 28)
	       (class . top-right-corner))
					 ))

       (transient-frame `(
	      ((background . ,t-top-left-images)
	       (left-edge . -5)
	       (top-edge . -5)
	       (class . top-left-corner))

	      ;;top-right corner
	      ((background . ,t-top-right-images)
	       (right-edge . -26)
	       (top-edge . -5)
	       (class . top-right-corner))

	      ;;title border
	      ((background . ,t-top-images)
	       (left-edge . -1)
	       (right-edge . 8)
	       (top-edge . -5)
	       (class . top-border))

	      ;; left border
	      ((background . ,t-left-images)
	       (left-edge . -5)
	       (top-edge . -1)
	       (bottom-edge . -1)
	       (class . left-border))

	      ;; right border
	      ((background . ,t-right-images)
	       (right-edge . -26)
	       (top-edge . 28)
	       (bottom-edge . 0)
	       (class . title))

	      ;; bottom border
	      ((background . ,t-bottom-images)
	       (left-edge . -1)
	       (right-edge . -2)
	       (bottom-edge . -5)
	       (class . bottom-border))

	      ;; bottom-left corner
	      ((background . ,t-bottom-left-images)
	       (left-edge . -5)
	       (bottom-edge . -5)
	       (class . bottom-left-corner))

	      ;; bottom-right corner
	      ((background . ,t-bottom-right-images)
	       (right-edge . -26)
	       (bottom-edge . -5)
	       (class . bottom-right-corner))

	      ;; delete button
	      ((background . ,t-close-images)
	       (right-edge . -26)
	       (top-edge . 0)
	       (class . close-button))

	      ;; green
	      ((background . ,t-right-green-images)
	       (right-edge . -26)
	       (top-edge . 28)
	       (height . ,(lambda (w) (green-height w)))
	       (class . title))

	      ;; curves
	      ((background . ,t-right-curves-images)
	       (right-edge . -26)
	       (top-edge . ,(lambda (w) (+ 28 (green-height w))))
	       (class . title))
				      ))

       (shaded-transient-frame `(
	       ((background . ,t-top-right-shaded-images)
		(right-edge . -26)
		(top-edge . -5)
		(class . top-right-corner))

	       ;; right border
	       ((background . ,t-right-images)
		(right-edge . -26)
		(top-edge . 1)
		(bottom-edge . -1)
		(class . title))

	       ;; bottom-right corner
	       ((background . ,t-bottom-right-shaded-images)
		(right-edge . -26)
		(bottom-edge . -5)
		(class . bottom-right-corner))

	       ;; delete button
	       ((background . ,t-close-images)
		(right-edge . -26)
		(top-edge . 0)
		(class . close-button))

	       ;; green
	       ((background . ,t-right-green-images)
		(right-edge . -26)
		(top-edge . 28)
		(height . ,(lambda (w) (green-height w)))
		(class . title))

	       ;; curves
	       ((background . ,t-right-curves-images)
		(right-edge . -26)
		(top-edge . ,(lambda (w) (+ 28 (green-height w))))
		(class . title))
				 ))
						     )
  
  (add-frame-style 'milliGun
		   (lambda (w type)
		     (cond ((eq type 'shaded)
			    shaded-frame)
			   ((eq type 'transient)
			    transient-frame)
			   ((eq type 'shaded-transient)
			    shaded-transient-frame)
			   ((eq type 'unframed)
			    nil-frame)
			   (t
			    frame))))

  (call-after-property-changed 'WM_NAME update-text-width)

  (mapc (lambda (sym)
	  (custom-set-property sym ':after-set (lambda() (rebuild-frames-with-style 'milliGun))))
	'(milliGun:percent-blue milliGun:percent-green milliGun:center-title))
  )
